package net.wanji.utc.hisense.netty.handler;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.socket.DatagramPacket;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.spring.ServiceBeanContext;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.utc.hisense.cache.CrossInfoCache;
import net.wanji.utc.hisense.cache.SignalDataCache;
import net.wanji.utc.hisense.cache.netty.NettyMessageCache;
import net.wanji.utc.hisense.common.enums.CommandResultSignEnum;
import net.wanji.utc.hisense.netty.commandsign.CommandResultSign;
import net.wanji.utc.hisense.netty.pojo.CommandPojo;
import net.wanji.utc.hisense.netty.response.CommandResponseFactory;
import net.wanji.utc.hisense.pojo.convert.RunningLightsStatusPojo;
import net.wanji.utc.hisense.pojo.netty.MessageResultPojo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.net.InetSocketAddress;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@Slf4j
@Component
public class NettyServerHandler extends SimpleChannelInboundHandler<DatagramPacket> {
    // 信号机ID和路口ID映射
    private static final Map<String, String> signalMap = new HashMap<>();
    @Autowired
    private CrossInfoMapper crossInfoMapper;

    @PostConstruct
    public void init() {
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectAll();
        for (CrossInfoPO crossInfoPO : crossInfoPOList) {
            String crossId = crossInfoPO.getId();
            String signalCode = crossInfoPO.getCode();
            signalMap.put(signalCode, crossId);
        }
    }

    @Resource
    private CrossInfoCache crossInfoCache;

    /**
     * 读取消息
     *
     * @param ctx
     * @param packet
     * @throws Exception
     */
    @Override
    protected void channelRead0(ChannelHandlerContext ctx, DatagramPacket packet) throws Exception {
        ByteBuf content = packet.content();
        InetSocketAddress remote = packet.sender();
        int i = content.readableBytes();
        byte[] b = new byte[i];
        content.readBytes(b);

        String data = new String(b, StandardCharsets.UTF_8);
        Object resultPojo = CommandResultSign.getHexSign(data);

        String className = CommandResultSignEnum.getResultPojo(resultPojo);
        if (CommandResultSignEnum.GET_RUNNING_STATE_INFO.getClassName().equals(className)) {
            // 直接set灯态缓存
            RunningLightsStatusPojo pojo = (RunningLightsStatusPojo) resultPojo;
            String signalCode = pojo.getCID();
            String crossId = signalMap.get(signalCode);
            Map<String, RunningLightsStatusPojo> cache = SignalDataCache.runningStateInfoCache;
            if (crossId != null) {
                cache.put(crossId, pojo);
            }
            return;
        }

        Object bean = ServiceBeanContext.getBean(className);
        CommandResponseFactory commandResponseFactory = ServiceBeanContext.getBean(className);

        String key = StringUtils.join("/", remote.getHostString(), ":", remote.getPort(), "/",
                resultPojo.getClass().getSimpleName());

        CommandPojo commandPojo = getCommandPojo(key, resultPojo);
        Object hexResult = commandResponseFactory.getCommandResponse(commandPojo);
        if (Objects.nonNull(hexResult)) {
            setResultData(key, hexResult);
        }
    }

    private void setResultData(String key, Object hexResult) {
        MessageResultPojo resultPojo = NettyMessageCache.NETTY_RESULT_TIMEOUT_MAP.get(key);
        if (resultPojo != null) {
            resultPojo.setHexMessageResult(hexResult);
            resultPojo.getCountDownLatch().countDown();
        }
    }

    private CommandPojo getCommandPojo(String key, Object data) {
        CrossInfoPO crossInfo = NettyMessageCache.COMMAND_RESULT_SIGN_MAP.get(key);
        CommandPojo commandPojo = new CommandPojo();
        commandPojo.setCrossId(crossInfo.getId());
        commandPojo.setSignalId(crossInfo.getCode());
        commandPojo.setSignalIp(crossInfo.getIp());
        commandPojo.setPort(crossInfo.getPort());
        commandPojo.setResponseMsg(data);
        return commandPojo;
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
    }
}
